package fxVuororekisteri;

import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import fi.jyu.mit.ohj2.Mjonot;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import org.apache.commons.lang3.tuple.Triple;
import vuororekisteri.Laji;
import vuororekisteri.Vuoro;
import vuororekisteri.Vuororekisteri;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Vuoron lisäys ja muokkaus ikkuna controlleri
 * Hoitaa muunmuassa oikeellisustarkistukset ja vuorojen lisäys ja muokkaukseen liittyvät toiminnot
 */
public class LisaaMuokkaaVuoroGUIController implements ModalControllerInterface<Triple<Vuororekisteri, Vuoro, StringBuilder>> {


    private Vuororekisteri vuororekisteri;
    private Vuoro vuoro;
    private StringBuilder muokattava;
    int vuoroId;


    @FXML
    private Button buttonPeruuta;

    @FXML
    private Button buttonTallenna;

    @FXML
    private TextField textAika;

    @FXML
    private TextField textLaji;

    @FXML
    private TextField textSijainti;

    @FXML
    private TextField textTaso;

    @FXML
    void handlePeruuta() {
        peruuta();
    }

    @FXML
    void handleTallenna(ActionEvent event) {
        tallenna(textAika, textLaji, textSijainti, textTaso);

    }


    // ================================================================================================================================


    /**
     * Jos halutaan muokata vuoroa niin vuoron tiedot täyttyvät kätevästi valmiiksi tekstikenttiin
     */
    public void asetaMuokkaus() {
        if ( muokattava != null ) {
            // int vuoroId = Integer.parseInt(Mjonot.erota(muokattava, ' '));

            for (int i = 0; i < vuororekisteri.getVuoroja(); i++) {
                Vuoro vuoro1 = vuororekisteri.annaVuoro(i);
                if ( vuoro1.getVuoroID() == vuoroId ) {
                    textLaji.setText(vuororekisteri.lajiIdPerusteella(vuoro1.getLajiID()).getNimi());
                    textAika.setText(String.valueOf(vuoro1.getAlkuaika()));
                    textSijainti.setText(vuororekisteri.lajiIdPerusteella(vuoro1.getLajiID()).getSijainti());
                    textTaso.setText(vuororekisteri.lajiIdPerusteella(vuoro1.getLajiID()).getTaso());

                }
            }
        }
    }


    private void peruuta() {

        ModalController.closeStage(buttonPeruuta);

    }

    /**
     * Tässä tarkistetaan että ei ole tölläistä vuoroa jo olemassa, eikä ajat ole älyttömät ja pitää olla laji olemassa
     * @param aika 22
     * @param laji 22
     * @param sijainti 22
     * @param taso 22
     * @return palauttaa true jos menee tarkastuksesta läpi false muuten
     */
    public boolean tarkistaVuoro(TextField aika, TextField laji, TextField sijainti, TextField taso) {

        double alkuaika = Double.parseDouble(String.valueOf(textAika.getText()));
        if ( !(alkuaika >= 0.0 && alkuaika <= 24.0) ) return false;

        Laji laji2 = vuororekisteri.haeLajiVuorolle(textLaji.getText(), textSijainti.getText(), textTaso.getText());
        if ( laji2 == null ) return false;
        else {
            int lajinid = laji2.getId();
            for (int i = 0; i < vuororekisteri.getVuoroja(); i++) {
                Vuoro vuoro2 = vuororekisteri.annaVuoro(i);
                if (vuoro2.getLajiID() == lajinid && vuoro2.getAlkuaika() == alkuaika) {
                    return false;
                }
            }
        }

        return true;
    }


    /**
     * Kun painetaan nappia tallenna niin vuoro rekisteröidään eli sille annetaan id
     * vuorolle asetetaan myös "leikki" arvot ellei ole kirjoitettu tekstikenttiin oikeita arvoja
     * @param aika Aika joka vuorolle halutaan antaa
     * @param laji lajin nimi jolle vuoro kuuluu
     * @param sijainti sijainti jossa laji on
     * @param taso vuoron taso
     */
    public void tallenna(TextField aika, TextField laji, TextField sijainti, TextField taso) {

        if (muokattava == null) {
            this.vuoro = new Vuoro();

            if (textLaji.getText().isEmpty() || textSijainti.getText().isEmpty() || textTaso.getText().isEmpty() || textAika.getText().isEmpty()) {
                if (!tarkistaVuoro(aika, laji, sijainti, taso)) {
                    Dialogs.showMessageDialog("tällänen vuoro on jo olemassa");
                }
                else {
                    vuoro.rekisteroiVuoro();
                    vuoro.asetaTesti();
                }
            }
            else {
                Laji lajiVuoroon = vuororekisteri.haeLajiVuorolle(textLaji.getText(), textSijainti.getText(), textTaso.getText());
                if (lajiVuoroon == null) {
                    Dialogs.showMessageDialog("ei ole tuollasta lajia olemassa");
                } else {
                    if (!tarkistaVuoro(aika, laji, sijainti, taso)) {
                        Dialogs.showMessageDialog("Tarkista alkuaika, ethän yrittänyt tehdä jo olemassa olevaa vuoroa");
                    }
                    else {
                        vuoro.rekisteroiVuoro();
                        vuoro.asetaTesti(lajiVuoroon.getId(), lajiVuoroon.getNimi(), lajiVuoroon.getSijainti(), lajiVuoroon.getTaso(), aika.getText());
                    }
                }
            }
        }

        else {

            int n = 0;
            for (int i = 0; i < vuororekisteri.getVuoroja(); i++) {
                Vuoro vuoro1 = vuororekisteri.annaVuoro(i);
                if ( vuoro1.getVuoroID() == vuoroId ) {
                    // Laji laji2 = vuororekisteri.lajiIdPerusteella(vuoro1.getLajiID());
                    if (!tarkistaVuoro(textAika, textLaji, textSijainti, textTaso)) {
                        Dialogs.showMessageDialog("Tälläien vuoro on jo olemassa, valitse joku toinen aika");
                    }
                    else {
                        vuoro1.muokkaaVuoroa(String.valueOf(textAika.getText()));
                    }
                }
            }
        }
        // Dialogs.showMessageDialog("Tälläien vuoro on jo olemassa, valitse joku toinen aika");
        ModalController.closeStage(buttonTallenna);
    }


    /**
     * Palauttaa triplen joka sisältää Vuoron joka ollaan lisätty tai muokattu, jos ei olla varattu tai muokattu mitään niin palautetaan uusi vuoro jonka id on nolla ja tiedot ovat muuttamattomia
     * @return -//-
     */
    @Override
    public Triple<Vuororekisteri, Vuoro, StringBuilder> getResult() {
        return new Triple<>() {
            @Override
            public Vuororekisteri getLeft() {
                return null;
            }

            @Override
            public Vuoro getMiddle() {
                return vuoro;
            }

            @Override
            public StringBuilder getRight() {
                return null;
            }
        };
    }

    /**
     * Asetetaan tuodun triplen arvoista ikkuna tuntemaan Vuororekisteri ja vuoron rivi jota halutaan muokata jos sellainen on valittu
     * @param tuonti Vuororekisteri, Vuoro, ja muokattavan vuoron rivi
     */
    @Override
    public void setDefault(Triple<Vuororekisteri, Vuoro, StringBuilder> tuonti) {
        this.vuororekisteri = tuonti.getLeft();
        this.muokattava = tuonti.getRight();
        if ( muokattava != null ) {
            this.vuoroId = Integer.parseInt(Mjonot.erota(muokattava, ' '));
            asetaMuokkaus();
        }

    }


    @Override
    public void handleShown() {

    }

}
