package fxVuororekisteri;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.ohj2.Mjonot;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TextArea;
import javafx.util.Pair;
import org.apache.commons.lang3.tuple.Triple;
import vuororekisteri.Laji;
import vuororekisteri.SailoException;
import vuororekisteri.Vuoro;
import vuororekisteri.Vuororekisteri;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.*;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Vuororekisterin pääikkunan controller luokka
 * Hoitaa pääikkunassa tapahtuvat toiminnot kuten suodattimet ja napit
 */
public class VuororekisteriGUIController {

    private Vuororekisteri vuororekisteri;

    @FXML
    private Button buttonValitse;

    @FXML
    private MenuItem menuLisaaLaji;

    @FXML
    private MenuItem menuLisaaVuoro;

    @FXML
    private MenuItem menuMuokkaaVuoroa;

    @FXML
    private MenuItem menuPoistaVuoro;

    @FXML
    private TextArea textLajit;

    @FXML
    private TextArea textVuorot;

    @FXML
    private ListChooser<String> textVuorotChooser;

    @FXML
    private ComboBoxChooser<String> chooserAika;

    @FXML
    private ComboBoxChooser<String> chooserLaji;

    @FXML
    private ComboBoxChooser<String> chooserSijainti;

    @FXML
    private ComboBoxChooser<String> chooserTaso;

    @FXML
    void avaaLajinLisays(ActionEvent event) {
         avaaikkuna(menuLisaaLaji);

    }

    @FXML
    void avaaVuoronLisays(ActionEvent event) {
        avaaikkuna(menuLisaaVuoro);
    }

    @FXML
    void avaaVuoronMuokkaus(ActionEvent event) {
        avaaikkuna(menuMuokkaaVuoroa);
    }

    @FXML
    void avaaVuoroPoisto(ActionEvent event) {
        avaaikkuna(menuPoistaVuoro);
    }

    @FXML
    void avaaVuoronVaraus(ActionEvent event) {
        avaaVuoronVarausikkuna();
    }

    @FXML
    void paivitaListaSuodatin(ActionEvent event) {
        kriteeri();
    }


    // ==========================================================================================================================================


    /**
     * asettaa ikkunan tunteman vuororekisterin tuoduksi
     * ja päivitetään samalla listChooser sekä suodattimet
     * @param vuororekisteri vuororekisteri joka tuodaan
     */
    public void setVuororekisteri(Vuororekisteri vuororekisteri) {
        this.vuororekisteri = vuororekisteri;
        alustaKentat();
        paivitaSuodattimet();
    }


    /**
     * alustetaan ohjelma avatessa tekstikenttiin tiedostossa olevien
     */
    public void alustaKentat() {
        String tiedNimiLajit = "lajitTesi.txt";
        String tiedVuorot = "vuorotTiedosto.txt";

        StringBuilder palautusLajit = new StringBuilder();
        StringBuilder palautusVuorot = new StringBuilder();

        try {
            vuororekisteri.lueLaji(tiedNimiLajit);
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }

        try (Scanner sc = new Scanner(new File(tiedNimiLajit))) {
            while (sc.hasNextLine()) {
                String rivi = sc.nextLine();

                if (rivi.isEmpty()) continue;

                StringBuilder rivi1 = new StringBuilder(rivi);
                int id = Integer.parseInt(Mjonot.erota(rivi1, '|'));
                String nimi = Mjonot.erota(rivi1, '|');
                String sijainti = Mjonot.erota(rivi1, '|');
                String taso = Mjonot.erota(rivi1, '|');

                palautusLajit.append(id + " " + nimi + " " + sijainti + " " + taso + "\n");
            }
            if (!palautusLajit.toString().isEmpty()) textLajit.setText(palautusLajit.toString());

        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }

        try {
            vuororekisteri.lueVuoro("vuorotTiedosto.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }
        try (Scanner scan = new Scanner(new File(tiedVuorot))) {
            while (scan.hasNextLine()) {
                String rivi = scan.nextLine();
                if (rivi.isEmpty()) continue;

                StringBuilder rivi1 = new StringBuilder(rivi);
                int lajiId = Integer.parseInt(Mjonot.erota(rivi1, '|'));
                int vuoroId = Integer.parseInt(Mjonot.erota(rivi1, '|'));
                double alkuAika = Double.parseDouble(Mjonot.erota(rivi1, '|'));
                double kesto = Double.parseDouble(Mjonot.erota(rivi1, '|'));
                boolean status = Boolean.parseBoolean(Mjonot.erota(rivi1, '|'));
                palautusVuorot.append(lajiId + " " + vuoroId + " " + alkuAika + " " + kesto + " " + status + "\n");
                Laji laji2 = vuororekisteri.lajiIdPerusteella(lajiId);
                if (laji2 != null && alkuAika != -1.0) {
                    if (status) {
                        textVuorotChooser.add(vuoroId + "   " + laji2.getNimi() + "   Taso: " + laji2.getTaso() + "     Sijainti: " + laji2.getSijainti() + "   Alkuaika: " + alkuAika + "  Kesto: " + kesto + "    Vapaa");
                    }
                    else {
                        textVuorotChooser.add(vuoroId + "   " + laji2.getNimi() + "   Taso: " + laji2.getTaso() + "     Sijainti: " + laji2.getSijainti() + "   Alkuaika: " + alkuAika + "  Kesto: " + kesto + "    Varattu");
                    }
                }
            }
            if (!palautusVuorot.toString().isEmpty()) {
                textVuorot.setText(palautusVuorot.toString());
            }
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * Aliohjelma joka katsoo choosereista valitut kriterit ja näyttää vain kriteerit täyttävät vuorot varaajalle
     * hakee kyseiset vuorot jo lajitellusta listasta.
     */
    public void kriteeri() {

        textVuorotChooser.clear();

        String valAika = String.valueOf(chooserAika.getValue());
        String valSijainti = String.valueOf(chooserSijainti.getValue());
        String valLaji = String.valueOf(chooserLaji.getValue());
        String valTaso = String.valueOf(chooserTaso.getValue());

        List<Vuoro> sorted4= lajitteleVuorot();

        for (int i = 0; i < sorted4.size(); i++) {
            Vuoro vuoro2 = sorted4.get(i);
            if (vuoro2.getLajiID() != 0) {
                Laji laji2 = vuororekisteri.lajiIdPerusteella(vuoro2.getLajiID());

                boolean aikaMatch = valAika.equals("Tyhjä") || Math.abs(Double.parseDouble(valAika) - vuoro2.getAlkuaika()) < 0.3;
                boolean sijaintiMatch = valSijainti.equals("Tyhjä") || laji2.getSijainti().equals(valSijainti);
                boolean lajiMatch = valLaji.equals("Tyhjä") || laji2.getNimi().equals(valLaji);
                boolean tasoMatch = valTaso.equals("Tyhjä") || laji2.getTaso().equals(valTaso);

                if (aikaMatch && sijaintiMatch && lajiMatch && tasoMatch && laji2 != null && vuoro2.getAlkuaika() != -1.0) {
                    tulostaChooser(vuoro2, laji2);
                }
            }
        }

    }


    /**
     * Suodattimista laji ja sijainti päivitetään niin että niistä löyttyy ajantasaiset lajin nimi ja sijainti tiedot valmiina valittavaksi
     */
    public void paivitaSuodattimet() {
        for (int i = 0; i < vuororekisteri.getLajeja(); i++) {
            String lajinNimi = vuororekisteri.annaLaji(i).getNimi();
            int n = 0;
            for (Object item : chooserLaji.getItems()) {
                if (lajinNimi.equals(item.toString()))
                    n++;
            }
            if (n == 0) chooserLaji.add(lajinNimi);

            String sijainti = vuororekisteri.annaLaji(i).getSijainti();
            int k = 0;
            for (Object item2 : chooserSijainti.getItems()) {
                if (sijainti.equals(item2.toString()))
                    k++;
            }
            if (k == 0) chooserSijainti.add(sijainti);

        }
    }


    /**
     * avaa halutun ikkunan riippuen siitä halutaanko painetaanko vuoron vai lajin lisäys nappia (if lause)
     * Tehdään ikkuan avaamisen yhteydessä uusi tyhjä laji tai vuoro
     * Ikkuna joka avataan palauttaa muokatun olion
     * olion kanssa mennään päivitä ali ohjalmaan jota käytetään olion tietojen esittämiseen/kirjoittamiseen pääikkunassa
     * @param avattava painike jota on painettu menussa ja halutaab avata
     */
    public void avaaikkuna(MenuItem avattava) {

        if (avattava == menuLisaaVuoro || avattava == menuMuokkaaVuoroa) {

            StringBuilder rivi;
            if (textVuorotChooser.getSelectedObject() != null) {
                rivi = new StringBuilder(textVuorotChooser.getSelectedObject());
            }
            else {
                rivi = null;
            }

            Triple<Vuororekisteri, Vuoro, StringBuilder> data =
                    Triple.of(vuororekisteri, null, rivi);

            Vuoro vuoro;
            data = ModalController.showModal(getClass().getResource("LisaaMuokkaaVuoroGUIView.fxml"), "Vuoron muokkaus", null, data);
            vuoro = data.getMiddle();
            if (vuoro != null) {
                paivitaVuoro(vuoro);
            }
            else {
                paivitaLista();
            }
            return;
        }

        if (avattava == menuLisaaLaji) {
            Laji laji = new Laji();
            var pari1 = new Pair<>(laji, vuororekisteri);
            pari1 = ModalController.showModal(getClass().getResource("LajinLisaysGUIView.fxml"), "Lisää laji", null, pari1);
            laji = pari1.getKey();
            if (laji.getId() != 0) {
                paivitaLaji(laji);
                paivitaSuodattimet();
            }
            return;
        }

        if (avattava == menuPoistaVuoro) {
            String valittu = textVuorotChooser.getSelectedObject();
            if (valittu != null) {
                poistaVuoro(valittu);
            }
        }
    }


    /**
     * avataan vuoronvarausikkuna ja tehdään siihen liittyvät esityöt
     * kun palataan ikkunasta takaisin niin päivitetään lista niin ettei siellä esimerkikis näy juuri poistettua vuoroa
     */
    public void avaaVuoronVarausikkuna() {
        String valittu = textVuorotChooser.getSelectedObject();
        StringBuilder rivi2 = new StringBuilder(valittu);
        int vuoronId = Integer.parseInt(Mjonot.erota(rivi2, ' '));
        Mjonot.erota(rivi2, ' ', true);
        if ( rivi2.toString().trim().equals("Varattu")) {
            Dialogs.showMessageDialog("Tämä vuoro on jo varattu!");
        }

        else {
            var pari2 = new Pair<>(vuoronId, vuororekisteri);
            ModalController.showModal(getClass().getResource("VuoronVarausGUIView.fxml"), "Vuoron varaus", null, pari2);
            // jos varaus onnistuu niin päivitetään listat / tarvitseeko edes vlttämättä tarkistaa onnistuminen?
            paivitaLista();
        }
    }


    /**
     * Aliohjelma joka kirjoittaa lajien tekstikkenttään rekiserin tiedot olemassa olevien ja rekisteröityjen lajien osalta
     * @param laji Laji joka palautuu kun tullaan pois vuoronlisäysikkunasta
     */
    public void paivitaLaji(Laji laji) {
        StringBuilder palautettava = new StringBuilder();

        try {
            vuororekisteri.lisaa(laji);
        } catch (SailoException e) {
            e.printStackTrace();
        }

        palautettava.append(laji.getId() + laji.getNimi() + laji.getSijainti() + laji.getTaso() + "\n");

        try {
            vuororekisteri.tallennaLaji("lajitTesi.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }
        System.out.println("Lajin nro: " + (laji.getId() - 1));
        laji.tulosta(System.out);

        textLajit.setText(textLajit.getText() + palautettava);
    }


    /**
     * Aliohjelma joka kirjoittaa vuorojen tekstikkenttään rekisterin tiedot olemassa olevien ja rekisteröityjen vuorojen osalta
     * @param vuoro Vuoro joka tulee vuoronlisäysikkunasta pois tullessa
     */
    public void paivitaVuoro(Vuoro vuoro) {
        StringBuilder palautettava = new StringBuilder();

        try {
            vuororekisteri.lisaaVuoro(vuoro);
        } catch (SailoException e) {
            e.printStackTrace();
        }

        for (int i=0; i<vuororekisteri.getVuoroja(); i++) {
            Vuoro vuoro1 = vuororekisteri.annaVuoro(i);

            if (vuoro.getStatus()) {
                palautettava.append(vuoro1.getLajiID() + " " + vuoro1.getVuoroID() + " " + vuoro1.getAlkuaika() + " " + vuoro1.getKesto() + " vapaa" + "\n");
            }
            else {
                palautettava.append(vuoro1.getLajiID() + " " + vuoro1.getVuoroID() + " " + vuoro1.getAlkuaika() + " " + vuoro1.getKesto() + " varattu" + "\n");
            }
        }

        try {
            vuororekisteri.tallennaVuoro("vuorotTiedosto.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }

        System.out.println("Vuoron nro: " + (vuoro.getVuoroID() - 1));
        vuoro.tulosta(System.out);

        List<Vuoro> sorted3 = lajitteleVuorot();

        textVuorotChooser.clear();
        for (int i = 0; i < sorted3.size(); i++) {
            Vuoro vuoro2 = sorted3.get(i);
            Laji laji2 = vuororekisteri.lajiIdPerusteella(vuoro2.getLajiID());
            if (laji2 != null && vuoro2.getAlkuaika() != -1.0) {
                tulostaChooser(vuoro2, laji2);
            }
        }
        textVuorot.setText(palautettava.toString());
    }


    /**
     * Kun uusi vuoro luodaan niin lisätään se samalla tiedostoon ja kirjoitetaan listChooseriin
     */
    public void paivitaLista() {

        try {
            vuororekisteri.tallennaVuoro("vuorotTiedosto.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }

        List<Vuoro> sorted = lajitteleVuorot();

        textVuorotChooser.clear();

        for (int i = 0; i < sorted.size(); i++) {
            Vuoro vuoro2 = sorted.get(i);
            Laji laji2 = vuororekisteri.lajiIdPerusteella(vuoro2.getLajiID());
            if (laji2 != null && vuoro2.getAlkuaika() != -1.0) {
                tulostaChooser(vuoro2, laji2);
            }
        }

    }


    /**
     * Lajittelee vuororekisterissä olevan vuorojen taulukon niin että alkiot ovat aikajärjestyksessä
     * ei siis lajittele varsinaisesti vuororekisterin listaa vaan tekee uuden jossa samat alkiot mutta järjestyksessä
     * @return Vuorot sisältävän taulukon aikajärjestetynä
     */
    public List<Vuoro> lajitteleVuorot() {
        List<Vuoro> lajitellut = new ArrayList<>();

        for (int i = 0; i < vuororekisteri.getVuoroja(); i++) {
            Vuoro vuoro5 = vuororekisteri.annaVuoro(i);
            lajitellut.add(vuoro5);
        }

        lajitellut.sort(Comparator.comparing(Vuoro::getAlkuaika));

        return lajitellut;
    }

    /**
     * poistaa valitun vuoron rekisteristä, eli valittua vuoroa ei enää ole liittymässä, rekisterissä eikä tiedostossa
     * ei kuitenkaan varsinaisesti poistu mistään mutta mitätöidään vuoron arvot niin että se ei näy käyttöliittymässä ja näin ollen sitä ei pysty käyttämään
     * @param poistettava Rivi joka on valittu ennen kun on painettu poista nappia
     */
    public void poistaVuoro(String poistettava) {
        StringBuilder rivi = new StringBuilder(poistettava);
        int poistettavanVuoronID = Integer.parseInt(Mjonot.erota(rivi, ' ' ));
        vuororekisteri.poistaVuoro(poistettavanVuoronID);

        try {
            vuororekisteri.tallennaVuoro("vuorotTiedosto.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }

        List<Vuoro> sorted2 = lajitteleVuorot();

        textVuorotChooser.clear();
        for (int i = 0; i < sorted2.size(); i++) {
            Vuoro vuoro2 = sorted2.get(i);
            Laji laji2 = vuororekisteri.lajiIdPerusteella(vuoro2.getLajiID());
            if (laji2 != null && vuoro2.getAlkuaika() != -1.0) {
                tulostaChooser(vuoro2, laji2);
            }
        }
    }


    /**
     * aliohjelma joka tulostaa listchooseriin tuodun lajin tiedot oikeassa muodossa
     * @param vuoro vuoro joka halutaan tulostaa
     * @param laji laji joka vuorolla on jotta saadaan lajin tiedot kirjoitettua chooseriin
     */
    public void tulostaChooser(Vuoro vuoro, Laji laji) {
        if (vuoro.getStatus()) {
            textVuorotChooser.add(vuoro.getVuoroID() + " " + laji.getNimi() + "   Taso: " + laji.getTaso() + "   Sijainti: " + laji.getSijainti() + "   Alkuaika: " + vuoro.getAlkuaika() + "  Kesto: " + vuoro.getKesto() + "    Vapaa");
        }
        else {
            textVuorotChooser.add(vuoro.getVuoroID() + " " + laji.getNimi() + "   Taso: " + laji.getTaso() + "   Sijainti: " + laji.getSijainti() + "   Alkuaika: " + vuoro.getAlkuaika() + "  Kesto: " + vuoro.getKesto() + "   Varattu");
        }
    }

}

