package vuororekisteri;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Tietää yksittäisen lajin tiedot ( id, nimi, sijainti ja taso)
 * Pystytään tarkistamaan tietyn kentän syntaksin oikeellisuus
 * Osaa muuttaa tiedostoon tallennetun muodon (1 | säbä | sisäpelikenttä | aloittelija) yksittäisiksi ominaisuuksiksi esim ensimmäisen tolpan jälkeinen osa enne toista tolppaa kertoo nimen
 * Osaa tehdä myös saman homman toiseen suuntaan
 * Osaa laittaa merkkijonon i kenttään halutun tiedon
 */
public class Laji {

    private int id;
    private String nimi;
    private String sijainti;
    private String taso;

    private static int seuraava = 1;


    public Laji() {
        id = 0;
        nimi = "";
        sijainti = "";
        taso = "";

    }


    public Laji(Laji toinen) {
        this.id = toinen.getId();
        this.nimi = toinen.getNimi();
        this.sijainti = toinen.getSijainti();
        this.taso = toinen.getTaso();
    }


    public Laji(int id, String nimi, String sijainti, String taso) {
        this.id = id;
        this.nimi = nimi;
        this.sijainti = sijainti;
        this.taso = taso;
        seuraava++;

    }

    /**
     * testipääohjelma
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Laji futis = new Laji();
        Laji uinti = new Laji();
        Laji saba = new Laji();
        futis.rekisteroi();
        uinti.rekisteroi();
        saba.rekisteroi();
        futis.tulosta(System.out);

        uinti.tulosta(System.out);
        saba.tulosta(System.out);

        futis.asetaTesti();
        futis.tulosta(System.out);

        uinti.asetaTesti();
        uinti.tulosta(System.out);

        saba.asetaTesti();
        saba.tulosta(System.out);

    }


    /**
     * @example <pre name="test">
     *         Laji futis = new Laji();
     *         Laji uinti = new Laji();
     *         futis.asetaTesti("Futis", "Brazil", "Copa America");
     *         uinti.asetaTesti("Uinti", "Uimahalli", "Vesikko");
     *         futis.getNimi() === "Futis";
     *         futis.getSijainti() === "Brazil";
     *         uinti.getNimi() === "Uinti";
     *         uinti.getTaso() === "Vesikko";
     * </pre>
     * Asettaa lajin tiedot syötteen perusteella
     * @param nimi1 lajille asetettava nimi
     * @param sijainti1 lajille asetettava sijainti
     * @param taso1 lajille asetettava taso
     */
    public void asetaTesti(String nimi1, String sijainti1, String taso1) {
        nimi = nimi1;
        sijainti = sijainti1;
        taso = taso1;
    }



    public int getId() {
        return id;
    }


    public String getNimi() {
        return nimi;
    }


    public String getSijainti() {
        return sijainti;
    }


    public String getTaso() {
        return taso;
    }

    /**
     * @example <pre name="test">
     *     #STATICIMPORT
     *     #import vuororekisteri.Laji;
     *         Laji futis = new Laji();
     *         Laji uinti = new Laji();
     *         futis.rekisteroi();
     *         uinti.rekisteroi();
     *         futis.getId() === 1;
     *         uinti.getId() === 2;
     *         uinti.getId() - futis.getId() === 1;
     * </pre>
     * tässä annetaan lajille id ja kasvatetaan seuraavaa joka tulee seuraavaksi luodun lajin id:ksi
     * @return palauttaa tarvittaessa id
     */
    public int rekisteroi() {
        this.id = seuraava;
        seuraava++;
        return id;

    }

    /**
     * @example <pre name="test">
     *         Laji futis = new Laji();
     *         Laji uinti = new Laji();
     *         futis.asetaTesti("Futis", "Brazil", "Copa America");
     *         uinti.asetaTesti("Uinti", "Uimahalli", "Vesikko");
     *         futis.toString() === "0|Futis|Brazil|Copa America";
     *         uinti.toString() === "0|Uinti|Uimahalli|Vesikko";
     * </pre>
     * @return paluttaa halutun lajin tiedot tolpalla erotettuna
     */
    @Override
    public String toString() {
        return "" + getId() + "|" + getNimi() + "|" + getSijainti() + "|" + getTaso();
    }

    /**
     * Tulostaa parametrina tulevaan tietovirtaan lajin tiedot
     * @param out Tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", this.id) + "  " + nimi);
        out.println(sijainti + "  " + taso);
    }

    /**
     * Tekekee uuden tietovirran parametrina tuotuun paikkaan
     * @param os Tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

    /**
     * asetetaan lajille "leikki" arvot jos ei ikkunassa kirjoiteta arvoja lajille
     * Pystytään testaamaan ohjelman toimintaa
     */
    public void asetaTesti() {
        nimi = "futis";
        sijainti = "kuokkala";
        taso = "tryhi";
    }






}



