package vuororekisteri;


import fi.jyu.mit.ohj2.Mjonot;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Scanner;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Lukee ja kirjoittaa Lajit (lajit.dat) -tiedostoon
 * Osaa lisätä ja poistaa lajeja
 * Osaa etsiä lajeja
 * Collabs: Laji-luokka
 */
public class Lajit {

    private int lkm = 0;
    private final int maxLkm = 10;
    private Laji[] alkiot = new Laji[maxLkm];

    public Lajit() {

    }

    /**
     * testipääohjelma
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Lajit lajit = new Lajit();

        Laji futis = new Laji(), saba = new Laji(), uinti = new Laji();
        futis.rekisteroi();    futis.asetaTesti();
        saba.rekisteroi();   saba.asetaTesti();
        uinti.rekisteroi();   uinti.asetaTesti();

        try {
            lajit.lisaa(futis);
            lajit.lisaa(saba);
            lajit.lisaa(uinti);

            System.out.println("========== Jäsenet testi ==============");

            for (int i=0; i<lajit.getLkm(); i++) {
                Laji laji = lajit.anna(i);
                System.out.println("Lajin nro: " + i);
                laji.tulosta(System.out);
            }

        } catch ( SailoException ex ) {
            System.out.println(ex.getMessage());
        }

        try {
            lajit.tallennaLaji("lajitTesi.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Lisää parametrinä tuodun lajin alkiot taulukkoon oikealle paikalle
     * Lisää lajin siis rekisteriin koska sieltä tätä kutsutaan
     * jos taulukko on täynnä niin tehdään uusi isompi taulukko niin mahtuu lisäämään
     * @param laji laji joka halutaan lisätä
     * @throws SailoException Tietorakenteessa tapahtuu virhe
     */
    public void lisaa(Laji laji) throws SailoException {

        if (lkm >= alkiot.length) {
            Laji[] uusiTaulukko = new Laji[lkm*2];
            for (int i=0; i<alkiot.length; i++) {
                Laji laji1 = new Laji(alkiot[i]);
                uusiTaulukko[i] = laji1;

            }
            uusiTaulukko[lkm] = laji;
            alkiot = uusiTaulukko;

        }
        else {
            alkiot[lkm] = laji;
        }
        lkm++;

    }


    public int getLkm() {
        return this.lkm;
    }

    /**
     * @example <pre name="test">
     *     #import vuororekisteri.Lajit;
     *     #import vuororekisteri.Laji;
     *         Lajit lajit = new Lajit();
     *         Laji futis = new Laji(), saba = new Laji(), uinti = new Laji();
     *         futis.rekisteroi();
     *         saba.rekisteroi();
     *         uinti.rekisteroi();
     *         futis.asetaTesti("Futis", "Brazil", "WC");
     *         saba.asetaTesti("Salibandy", "Mallopylly", "SM");
     *         uinti.asetaTesti("Uinti", "USA", "Olympia");
     *         try {
     *             lajit.lisaa(futis);
     *             lajit.lisaa(saba);
     *             lajit.lisaa(uinti);
     *         } catch ( SailoException ex ) {
     *             System.out.println(ex.getMessage());
     *         }
     *         lajit.anna(2).getNimi() === "Uinti";
     *         lajit.getLkm() === 3;
     * </pre>
     * Palauttaa parametrinä tuodun indeksin paikalla olevan lajin
     * @param i indeksi josta laji annetaan
     * @return palauttaa indeksistö löytyneen lajin
     * @throws IndexOutOfBoundsException Laiton indeksi
     */
    public Laji anna(int i) throws IndexOutOfBoundsException {

        if (i < 0 || lkm <= i) {
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        }
        return alkiot[i];
    }


    /**
     * Lukee lajien tiedoston ja lisää siellä esiintyvät lajit lajien taulukkoon ohjelma käynnistäessä
     * @param tiedNimi tiedoston nimi josta luetaan
     * @throws SailoException Tiedostosta luku epäonnistuu / tiedostoa ei löydy
     *
     */
    public void lueLaji(String tiedNimi) throws SailoException {
        try (Scanner sc = new Scanner(new File(tiedNimi))) {
            while (sc.hasNextLine()) {
                String rivi = sc.nextLine();
                if (rivi.equals("null")) continue;
                if (rivi.isEmpty()) continue;

                StringBuilder rivi1 = new StringBuilder(rivi);
                int id = Integer.parseInt(Mjonot.erota(rivi1, '|'));
                String nimi = Mjonot.erota(rivi1, '|');
                String sijainti = Mjonot.erota(rivi1, '|');
                String taso = Mjonot.erota(rivi1, '|');

                Laji laji = new Laji(id, nimi, sijainti, taso);
                lisaa(laji);

            }
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * Tallentaa lajin luomisen yhteydessä lajit tiedostoon
     * @param tiedNimi tiedosto johon tallennetaan
     * @throws SailoException Tiedostoon ei voi tallentaa / tiedostoa ei löydy
     */
    public void tallennaLaji(String tiedNimi) throws SailoException {
        File tiedostoLajit = new File(tiedNimi);
        try (PrintStream fo = new PrintStream(new FileOutputStream(tiedostoLajit, false))){
            for (int i=0; i<lkm; i++) {
                Laji tallennettava = anna(i);

                fo.println(tallennettava);

            }
        }
        catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }


}
