package vuororekisteri;

import java.io.OutputStream;
import java.io.PrintStream;
import java.math.BigDecimal;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Haliitsee yksittäiseen vuoroon liittyvät toimenpiteet kuten luominen ja erilaiset muokkaukset
 */
public class Vuoro {
    // Atribuuttit
    private int lajiID;
    private int vuoroID;
    private double alkuAika;
    private double kesto;
    private boolean status; // onko vuoro vapaa vai ei
    private static int seuraava = 1;

    /**
     * Asettaa vuoron tiedot "nollaksi"
     * kesto on aina tunnin ja vuoro on vapaa
     */
    public Vuoro() {
        lajiID = 0;
        vuoroID = 0;
        alkuAika = 0.0;
        kesto = 1.0;  // kaikki vuorot kestää tunnin
        status = true;   // vuoro on luodessa vapaa
    }


    public Vuoro(int lajiid, int vuoroid, double alkuAika, double kesto, boolean status) {
        this.lajiID = lajiid;
        this.vuoroID = vuoroid;
        this.alkuAika = alkuAika;
        this.kesto = kesto;
        this.status = status;
        seuraava++;
    }


    /**
     * testipääohjelma
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Vuoro vuoro1 = new Vuoro();
        Vuoro vuoro2 = new Vuoro();
        Vuoro vuoro3 = new Vuoro();

        vuoro1.rekisteroiVuoro();
        vuoro2.rekisteroiVuoro();
        vuoro3.rekisteroiVuoro();

        vuoro1.tulosta(System.out);
        vuoro2.tulosta(System.out);
        vuoro3.tulosta(System.out);

        vuoro1.asetaTesti();
        vuoro1.tulosta(System.out);

        vuoro2.asetaTesti();
        vuoro2.tulosta(System.out);

        vuoro3.asetaTesti();
        vuoro3.tulosta(System.out);

    }


    public int getLajiID() {
        return lajiID;
    }

    public int getVuoroID() {
        return vuoroID;
    }


    public double getAlkuaika() {
        return alkuAika;
    }


    public double getKesto() {
        return kesto;
    }


    public boolean getStatus() {
        return status;
    }


    /**
     * @example <pre name="test">
     *     #STATICIMPORT
     *     #import vuororekisteri.Vuoro;
     *         Vuoro vuoro1 = new Vuoro();
     *         Vuoro vuoro2 = new Vuoro();
     *         vuoro1.rekisteroiVuoro();
     *         vuoro2.rekisteroiVuoro();
     *         vuoro1.getVuoroID() === 1;
     *         vuoro2.getVuoroID() === 2;
     *         vuoro2.getVuoroID() - vuoro1.getVuoroID() === 1;
     * </pre>
     * Annetaan vuorolle id ja kasvatetaan seuraavaksi annettavaa id:tä yhdellä
     * Palauttaa myös annetun id jota voidaan käyttää tarvittaessa
     * @return vuoron id
     */
    public int rekisteroiVuoro() {
        this.vuoroID = seuraava;
        seuraava++;
        return vuoroID;

    }


    /**
     * @example <pre name="test">
     *     #STATICIMPORT
     *         Vuoro vuoro1 = new Vuoro();
     *         Vuoro vuoro2 = new Vuoro();
     *         vuoro1.setVuoronArvot(19, 45);
     *         vuoro2.setVuoronArvot(12, 4);
     *         vuoro1.getAlkuaika() ~~~ 19.0;
     *         vuoro2.getAlkuaika() ~~~ 12.0;
     *         vuoro2.getKesto() ~~~ 4.0;
     * </pre>
     * Asettaa vuoron arvot parametreina tuoduiksi
     * käytetään "poistamisen" yhteydessä kun halutaan mitätöidä vuoro eli antaa alkuajaksi -1 (mahdoton) ja kesto 0
     * @param alkuaika aika joka halutaan antaa
     * @param kesto1 kesto koka halutaan antaa
     */
    public void setVuoronArvot(double alkuaika, double kesto1) {
        this.alkuAika = alkuaika;
        this.kesto = kesto1;
    }


    /**
     * asettaa vuoron statuksen falseksi eli varaa vuoron
     */
    public void varaaVuoro() {
        this.status = false;
    }


    /**
     * @example <pre name="test">
     *     #STATICIMPORT
     *         Vuoro vuoro1 = new Vuoro();
     *         Vuoro vuoro2 = new Vuoro();
     *         vuoro1.setVuoronArvot(19, 45);
     *         vuoro2.setVuoronArvot(12, 4);
     *         vuoro1.muokkaaVuoroa("16");
     *         vuoro1.getAlkuaika() ~~~ 16.0;
     *         vuoro2.getAlkuaika() ~~~ 12.0;
     *         vuoro2.getKesto() ~~~ 4.0;
     *         vuoro1.getKesto() ~~~ 45.0;
     * </pre>
     * Muokkaa vuoron eli pystyy muokkaamaan halutun vuoron alkuajan
     * @param aika aika joka vuorolle annetaan
     */
    public void muokkaaVuoroa(String aika) {
        this.alkuAika = Double.parseDouble(aika);
    }


    /**
     * Asetetaan vuorolle "leikki" arvot että nähdään että ohjelma toimii
     */
    public void asetaTesti() {
        lajiID = 1;
        alkuAika = 10.00;
        status = false;
    }


    /**
     * @example <pre name="test">
     *     #STATICIMPORT
     *         Vuoro vuoro1 = new Vuoro();
     *         Vuoro vuoro2 = new Vuoro();
     *         vuoro1.asetaTesti(10, "Curling", "Canada", "Olympi", "10");
     *         vuoro2.asetaTesti(14, "Curg", "ada", "ympi", "17");
     *         vuoro2.getAlkuaika() ~~~ 17.0;
     *         vuoro1.getLajiID() ~~~ 10;
     * </pre>
     * Asettaa vuoron luomisen yhteydessä lajiid jotta vuoro osataan yhdistää johonkin lajiin sekä antaa alkuajaksi kirjoitetun alkuajan
     * @param lajiId Lajiid joka vuorolle tulee syöteen perusteella
     * @param laji
     * @param sijainti"
     * @param taso ""
     * @param aika alkuaika joka ikkunaan on syöttetty
     */
    public void asetaTesti(int lajiId, String laji, String sijainti, String taso, String aika) {

        lajiID = lajiId;

        alkuAika = new BigDecimal(aika).doubleValue();

    }


    /**
     * @example <pre name="test">
     *     #STATICIMPORT
     *         Vuoro vuoro1 = new Vuoro();
     *         Vuoro vuoro2 = new Vuoro();
     *         vuoro1.setVuoronArvot(19, 45);
     *         vuoro1.toString() === "0|0|19.0|45.0|true";
     *         vuoro2.setVuoronArvot(12, 4);
     *         vuoro1.muokkaaVuoroa("16");
     *         vuoro1.toString() === "0|0|16.0|45.0|true";
     *         vuoro2.toString() === "0|0|12.0|4.0|true";
     * </pre>
     * @return palauttaa halutun vuoron tiedot tolpalla erotettuna
     */
    @Override
    public String toString() {
        return "" + getLajiID() + "|" + getVuoroID() + "|" + getAlkuaika() + "|" + getKesto() + "|" + getStatus();
    }


    /**
     * Tulostaa vuoron tiedot parametrina tulevaan tietovirtaan
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println("lajin id on: " + String.format("%03d", this.lajiID) + " " + "vuoron id on: " + String.format("%03d", this.vuoroID));
        out.println("alkuaika, kesto ja status: " + alkuAika + "  " + kesto + "  " + status);
    }

    /**
     * Tekee uuden printsteamin parametrina tuotuun paikkaan
     * @param os paikka johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

}
