package vuororekisteri;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Huolehti Lajit ja Vuorot luokkien välisestä yhteistyöstä ja välittää näitä tietoja pyydettäesä
 * Lukee ja kirjoittaa Vuororekisterin molempii (lajit ja vuorot) tiedostoon pyytämällä
 * Collabs: Laji, Lajit, Vuoro, Vuorot
 */
public class Vuororekisteri {

    private final Lajit lajit = new Lajit();
    private final Vuorot vuorot = new Vuorot();


    /**
     * testipääohjelma
     * @param args ei käytössä
     */
    public static void main(String[] args) {


        Vuororekisteri vuororekisteri = new Vuororekisteri();

        try {

            Laji futis = new Laji(), saba = new Laji(), uinti = new Laji();
            futis.rekisteroi();
            saba.rekisteroi();
            uinti.rekisteroi();

            futis.asetaTesti();
            saba.asetaTesti();
            uinti.asetaTesti();

            vuororekisteri.lisaa(futis);
            vuororekisteri.lisaa(saba);
            vuororekisteri.lisaa(uinti);

            vuororekisteri.tallennaLaji("lajitTesi.txt");

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < vuororekisteri.getLajeja(); i++) {
                Laji laji = vuororekisteri.annaLaji(i);
                System.out.println("Laji paikassa: " + i);
                laji.tulosta(System.out);
            }

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }


        try {

            Vuoro vuoro1 = new Vuoro(), vuoro2 = new Vuoro(), vuoro3 = new Vuoro();
            vuoro1.rekisteroiVuoro();
            vuoro2.rekisteroiVuoro();
            vuoro3.rekisteroiVuoro();

            vuoro1.asetaTesti();
            vuoro2.asetaTesti();
            vuoro3.asetaTesti();

            vuororekisteri.lisaaVuoro(vuoro1);
            vuororekisteri.lisaaVuoro(vuoro2);
            vuororekisteri.lisaaVuoro(vuoro3);

            vuororekisteri.tallennaVuoro("vuorotTiedosto.txt");

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < vuororekisteri.getVuoroja(); i++) {
                Vuoro vuoro = vuororekisteri.annaVuoro(i);
                System.out.println("Vuoro paikassa: " + i);
                vuoro.tulosta(System.out);
            }

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }

    }

    /**
     * lisätään parametrina tuotu laji, lajien taulukkoon
     * @param laji laji joka lisätään
     * @throws SailoException virhe tietorakenteessa
     */
    public void lisaa(Laji laji) throws SailoException {
        lajit.lisaa(laji);
    }

    /**
     * lisätään parametrina tuotu vuoro, vuorojen listaan
     * @param vuoro lisättävä vuoro
     * @throws SailoException virhe tietorekenteessa
     */
    public void lisaaVuoro(Vuoro vuoro) throws SailoException {
        vuorot.lisaaVuoro(vuoro);
    }


    /**
     * palauttaa rekisterissä olevien lajien lukumäärän
     * @return palauttaa rekisterissä olevien lajien lukumäärän
     */
    public int getLajeja() {
        return lajit.getLkm();
    }


    /**
     * palauttaa rekisterissä olevien vuorojen määrän
     * @return palauttaa rekisterissä olevien vuorojen määrän
     */
    public int getVuoroja() {
        return vuorot.getVuorotLkm();
    }

    /**
     * antaa parametrina tuodun indeksin mukaisen lajin
     * @param i indeksi
     * @return palauttaa lajin
     * @throws IndexOutOfBoundsException indeksi virhe
     */
    public Laji annaLaji(int i) throws IndexOutOfBoundsException{
        return lajit.anna(i);
    }

    /**
     * antaa parametrina tuodun indeksin mukaisen vuoron
     * @param i indeksi
     * @return paluttaa vuoron
     * @throws IndexOutOfBoundsException indeksi virhe
     */
    public Vuoro annaVuoro(int i) throws IndexOutOfBoundsException{
        return vuorot.annaVuoro(i);
    }

    /**
     * @example <pre name="test">
     *    Vuororekisteri vuororekisteri = new Vuororekisteri();
     *     try {
     *     Laji futis = new Laji(), saba = new Laji(), uinti = new Laji();
     *     futis.rekisteroi();
     *     saba.rekisteroi();
     *     uinti.rekisteroi();
     *     futis.asetaTesti("ViininMaistelu", "Ruotti", "Simp");
     *     saba.asetaTesti("Juustonmaistelu", "Porvoo", "Hienostelija");
     *     uinti.asetaTesti("Eukonkanto", "Lapua", "SM");
     *     vuororekisteri.lisaa(futis);
     *     vuororekisteri.lisaa(saba);
     *     vuororekisteri.lisaa(uinti);
     *     } catch (SailoException ex) {
     *     System.out.println(ex.getMessage());
     *     }
     *     vuororekisteri.haeLajiVuorolle("Juustonmaistelu", "Porvoo", "Hienostelija").getTaso() === "Hienostelija";
     *     vuororekisteri.haeLajiVuorolle("Juustonmaistelu", "Lapua", "Hienostelija") === null;
     * </pre>
     * Palauttaa lajin jolle halutaan tehdä vuoro
     * jos ei kirjeitettuja tietoja vastaavaa lajia löydy niin palautetaan null
     * Hakee lajin tuotujen tekstimuotoisten parametrien perusteella
     * @param laji ikkunaan annettu lajin nimi
     * @param sijainti ikkunaan annettu sijainti
     * @param taso ikkunaan annettu taso
     * @return laji joka vastaa syötettä tai jos ei löydy sellaista niin null
     */
    public Laji haeLajiVuorolle( String laji, String sijainti, String taso ) {

        for (int i = 0; i <getLajeja(); i++) {
            Laji laji1 = annaLaji(i);
            if (laji1.getNimi().equals(laji) && laji1.getTaso().equals(taso) && laji1.getSijainti().equals(sijainti)) {
                return annaLaji(i);
            }
        }
        return null;
    }


    /**
     * @example <pre name="test">
     *    Vuororekisteri vuororekisteri = new Vuororekisteri();
     *     try {
     *     Laji futis = new Laji(), saba = new Laji(), uinti = new Laji();
     *     futis.rekisteroi();
     *     saba.rekisteroi();
     *     uinti.rekisteroi();
     *     futis.asetaTesti("ViininMaistelu", "Ruotti", "Simp");
     *     saba.asetaTesti("Juustonmaistelu", "Porvoo", "Hienostelija");
     *     uinti.asetaTesti("Eukonkanto", "Lapua", "SM");
     *     vuororekisteri.lisaa(futis);
     *     vuororekisteri.lisaa(saba);
     *     vuororekisteri.lisaa(uinti);
     *     } catch (SailoException ex) {
     *     System.out.println(ex.getMessage());
     *     }
     *     vuororekisteri.lajiIdPerusteella(3).getNimi() === "Eukonkanto";
     *     vuororekisteri.lajiIdPerusteella(1).getTaso() === "Simp";
     * </pre>
     * Hakee rekisteristä tuodun id perusteella sitä vastaavan lajin
     * @param id lajin id joka vuorolla on
     * @return palauttaa lajin jolla on parametrina tuotu id numero jos ei löydy niin palauttaa null
     */
    public Laji lajiIdPerusteella(int id) {
        for (int i = 0; i <getLajeja(); i++) {
            Laji laji1 = annaLaji(i);
            if ( laji1.getId() == id ) {
                return laji1;
            }
        }
        return null;
    }


    /**
     * parametrina tuodun id:n perusteella etsii vuoron ja poistaa sen jos sellainen löytyy
     * @param id vuoron id joka halutaan poistaa
     */
    public void poistaVuoro(int id) {
        vuorot.poistaVuoro(id);
    }


    /**
     * Tallentaa vuororekisterissä olevat lajit tiedostoon joka tuodaan parametrina
     * @param tiedNimi tiedoston nimi johon tallennetaan
     * @throws SailoException Tiedostoa ei löydy tai virhe tietorakenteessa
     */
    public void tallennaLaji(String tiedNimi) throws SailoException {
        lajit.tallennaLaji(tiedNimi);
    }


    /**
     * Tallentaa vuororekisterissä olevat vuorot tiedostoon joka tuodaan parametrina
     * @param tiedNimi Tiedoston nimi johon tallennetaan
     * @throws SailoException Tiedostoa ei löydy tai virhe tietorakenteessa
     */
    public void tallennaVuoro(String tiedNimi) throws SailoException {
        vuorot.tallennaVuoro(tiedNimi);
    }


    /**
     * Lukee lajit parametrina tuodusta tiedostosta ja lisää ne rekisteriin
     * @param tiedNimi tiedosto josta luetaan
     * @throws SailoException Tiedostoa ei löydy tai virhe tietorakenteessa
     */
    public void lueLaji(String tiedNimi) throws SailoException {
        lajit.lueLaji(tiedNimi);
    }


    /**
     * Lukee vuorot parametrina tuodusta tiedostosta ja tallentaa ne rekisteriin
     * @param tiedNimi tiedosto josta luetaan
     * @throws SailoException Tiedostoa ei löydy tai virhe tietorakenteessa
     */
    public void lueVuoro(String tiedNimi) throws SailoException {
        vuorot.lueVuoro(tiedNimi);
    }

}
