package vuororekisteri;

import fi.jyu.mit.ohj2.Mjonot;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * @author Elias Haverinen, Lauri Vepsäläinen
 * @version 4.4.2025
 * Hallitsee vuorot kokonaisuutta eli muunmuassa listan käsittelyä
 */
public class Vuorot {

    private int vuorotLkm = 0;
    private List<Vuoro> vuoroAlkiot = new ArrayList<>();


    public Vuorot() {

    }

    /**
     * testipääohjelma
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Vuorot vuorot = new Vuorot();

        Vuoro vuoro1 = new Vuoro(), vuoro2 = new Vuoro(), vuoro3 = new Vuoro();
        vuoro1.rekisteroiVuoro();    vuoro1.asetaTesti();
        vuoro2.rekisteroiVuoro();   vuoro2.asetaTesti();
        vuoro3.rekisteroiVuoro();   vuoro3.asetaTesti();

        try {
            vuorot.lisaaVuoro(vuoro1);
            vuorot.lisaaVuoro(vuoro2);
            vuorot.lisaaVuoro(vuoro3);

            System.out.println("========== Jäsenet testi ==============");

            for (int i=0; i<vuorot.getVuorotLkm(); i++) {
                Vuoro vuoro = vuorot.annaVuoro(i);
                System.out.println("Vuoron nro: " + i);
                vuoro.tulosta(System.out);
            }

        } catch ( SailoException ex ) {
            System.out.println(ex.getMessage());
        }


        try {
            vuorot.tallennaVuoro("vuorotTiedosto.txt");
        } catch (SailoException e) {
            throw new RuntimeException(e);
        }

    }

    /**
     * Lisää vuoron vuorojen listaan
     * Lisää käytännössä vuoron rekisteriin koska sieltä tätä kutsutaan
     * @param vuoro Vuoro joka halutaan lisätä rekisteriin (vuorot listaan)
     * @throws SailoException Virhe tietorakenteessa
     */
    public void lisaaVuoro(Vuoro vuoro) throws SailoException {

        vuoroAlkiot.add(vuoro);
        this.vuorotLkm++;
    }


    /**
     * @example <pre name="test">
     *         Vuorot vuorot = new Vuorot();
     *         Vuoro vuoro1 = new Vuoro(), vuoro2 = new Vuoro(), vuoro3 = new Vuoro();
     *         vuoro1.rekisteroiVuoro();    vuoro1.asetaTesti(1, "Futis", "Helsinki", "HesaCup", "18");
     *         vuoro2.rekisteroiVuoro();   vuoro2.asetaTesti(2, "Säbä", "vantaa", "hohohehfc", "12");
     *         vuoro3.rekisteroiVuoro();   vuoro3.asetaTesti(3, "Uinti", "USA", "Vesikko", "7");
     *         try {
     *             vuorot.lisaaVuoro(vuoro1);
     *             vuorot.lisaaVuoro(vuoro2);
     *             vuorot.lisaaVuoro(vuoro3);
     *         } catch ( SailoException ex ) {
     *             System.out.println(ex.getMessage());
     *         }
     *         vuorot.annaVuoro(1).getLajiID() === 2;
     *         vuorot.annaVuoro(0).getAlkuaika() ~~~ 18.0;
     *         vuorot.poistaVuoro(2);
     *         vuorot.annaVuoro(1).getAlkuaika() ~~~ -1.0;
     * </pre>
     * etsii parametrina tuodun vuoroid perusteella poistettavan vuoron ja kutsuu setVuoronArvot aliohjelmaa joka "mitätöi" sen
     * @param id Poistettavan vuoron id
     */
    public void poistaVuoro(int id) {
        for (Vuoro vuoro : vuoroAlkiot) {
            if (id == vuoro.getVuoroID()) {
                vuoro.setVuoronArvot(-1.0, 0.0);
            }
        }
    }


    /**
     * palauttaa vuorot taulukon pituuden / vuorojen määrän
     * @return palauttaa vuorot taulukon pituuden / vuorojen määrän
     */
    public int getVuorotLkm() {
        return this.vuoroAlkiot.size();
    }

    /**
     * @example <pre name="test">
     *         Vuorot vuorot = new Vuorot();
     *         Vuoro vuoro1 = new Vuoro(), vuoro2 = new Vuoro(), vuoro3 = new Vuoro();
     *         vuoro1.rekisteroiVuoro();    vuoro1.asetaTesti(1, "Futis", "Helsinki", "HesaCup", "18");
     *         vuoro2.rekisteroiVuoro();   vuoro2.asetaTesti(2, "Säbä", "vantaa", "hohohehfc", "12");
     *         vuoro3.rekisteroiVuoro();   vuoro3.asetaTesti(3, "Uinti", "USA", "Vesikko", "7");
     *         try {
     *             vuorot.lisaaVuoro(vuoro1);
     *             vuorot.lisaaVuoro(vuoro2);
     *             vuorot.lisaaVuoro(vuoro3);
     *         } catch ( SailoException ex ) {
     *             System.out.println(ex.getMessage());
     *         }
     *         vuorot.annaVuoro(1).getLajiID() === 2;
     *         vuorot.annaVuoro(0).getAlkuaika() ~~~ 18.0;
     * </pre>
     * Antaa parametrina tuodun indeksin perusteella vuoron listasta
     * @param i indeksi josta vuoro annetaan listasta
     * @return palauttaa indeksiä vastaavan vuoron
     * @throws IndexOutOfBoundsException indeksillä ei löydy vuoroa
     */
    public Vuoro annaVuoro(int i) throws IndexOutOfBoundsException {

        if (i < 0 || vuoroAlkiot.size() <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return vuoroAlkiot.get(i);
    }


    /**
     * Lukee vuoro tiedostoa ja lisää sieltä luetut vuorot rekisteriin ( ei tee tätä itse siis)
     * @param tiedNimi tiedosto josta vuorot luetaan
     * @throws SailoException Tiedostoa ei löydy tai virhe tietorakenteessa
     */
    public void lueVuoro(String tiedNimi) throws SailoException {
        try (Scanner sc = new Scanner(new File(tiedNimi))) {
            while (sc.hasNextLine()) {
                String rivi = sc.nextLine();

                if (rivi.isEmpty()) continue;

                StringBuilder rivi1 = new StringBuilder(rivi);
                int lajiId = Integer.parseInt(Mjonot.erota(rivi1, '|'));
                int vuoroId = Integer.parseInt(Mjonot.erota(rivi1, '|'));
                double alkuAika = Double.parseDouble(Mjonot.erota(rivi1, '|'));
                double kesto = Double.parseDouble(Mjonot.erota(rivi1, '|'));
                boolean status = Boolean.parseBoolean(Mjonot.erota(rivi1, '|'));

                Vuoro vuoro = new Vuoro(lajiId, vuoroId, alkuAika, kesto, status);
                lisaaVuoro(vuoro);

            }
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * kirjoittaa vuorot tiedostoon
     * @param tiedNimi tiedosto johon kirjoitetaan
     * @throws SailoException Tiedostoa ei löydy tai virhe tietorakenteessa
     */
    public void tallennaVuoro(String tiedNimi) throws SailoException {
        File tiedostoVuorot = new File(tiedNimi);
        try (PrintStream fo = new PrintStream(new FileOutputStream(tiedostoVuorot, false))){
            for (int i=0; i<vuorotLkm; i++) {
                Vuoro tallennettava = annaVuoro(i);
                fo.println(tallennettava);
            }
        }
        catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }
}
